@@ -1,27 +1,66 @@
-mediation<-function(y,d,m,x,w=NULL,trim=0.05, ATET=FALSE, logit=FALSE){
+mediation<-function(y,d,m,x,w=NULL,s=NULL,z=NULL, selpop=FALSE, trim=0.05, ATET=FALSE, logit=FALSE){
   if (is.null(w)==TRUE){
     if (logit==FALSE){
-      pscore.mx=glm(d~cbind(m,x),family=binomial(probit))$fitted
-      pscore.x=glm(d~x,family=binomial(probit))$fitted
+      if (is.null(z)==TRUE){
+        pscore.mx=glm(d~cbind(m,x),family=binomial(probit))$fitted
+        pscore.x=glm(d~x,family=binomial(probit))$fitted
+      }
+      if (is.null(s)==FALSE & is.null(z)==TRUE) pscore.s=glm(s~cbind(d,m,x),family=binomial(probit))$fitted
+      if (is.null(s)==FALSE & is.null(z)==FALSE) {
+        pscore.s=glm(s~cbind(d,m,x,z),family=binomial(probit))$fitted
+        pscore.mx=glm(d~cbind(m,x,pscore.s),family=binomial(probit))$fitted
+        pscore.x=glm(d~cbind(x,pscore.s),family=binomial(probit))$fitted
+      }
     }
     if (logit==TRUE){
-      pscore.mx=glm(d~cbind(m,x),family=binomial(logit))$fitted
-      pscore.x=glm(d~x,family=binomial(logit))$fitted
+      if (is.null(z)==TRUE){
+        pscore.mx=glm(d~cbind(m,x),family=binomial(logit))$fitted
+        pscore.x=glm(d~x,family=binomial(logit))$fitted
+      }
+      if (is.null(s)==FALSE & is.null(z)==TRUE) pscore.s=glm(s~cbind(d,m,x),family=binomial(logit))$fitted
+      if (is.null(s)==FALSE & is.null(z)==FALSE) {
+        pscore.s=glm(s~cbind(d,m,x,z),family=binomial(logit))$fitted
+        pscore.mx=glm(d~cbind(m,x,pscore.s),family=binomial(logit))$fitted
+        pscore.x=glm(d~cbind(x,pscore.s),family=binomial(logit))$fitted
+      }
+    }
+    if (is.null(s)==TRUE | selpop==TRUE){
+      ind=((pscore.mx<trim) | (pscore.mx>(1-trim)) )
+      y=y[ind==0]; d=d[ind==0]; pscore.mx=pscore.mx[ind==0]; pscore.x=pscore.x[ind==0]
+    }
+    if (is.null(s)==FALSE & selpop==FALSE) {
+      ind=((pscore.mx<trim) | (pscore.mx>(1-trim)) | pscore.s<trim)
+      y=y[ind==0]; d=d[ind==0]; s=s[ind==0]; pscore.mx=pscore.mx[ind==0]; pscore.x=pscore.x[ind==0]; pscore.s=pscore.s[ind==0]
+    }
+    if ((is.null(s)==TRUE) | (is.null(s)==FALSE & selpop==TRUE)){
+      if (is.null(s)==FALSE & selpop==TRUE) {y=y[s==1]; d=d[s==1]; pscore.mx=pscore.mx[s==1]; pscore.x=pscore.x[s==1]}
+      if (ATET==FALSE){
+        y1m1=sum(y*d/pscore.x)/sum(d/pscore.x)
+        y1m0=sum(y*d*(1-pscore.mx)/(pscore.mx*(1-pscore.x)))/sum(d*(1-pscore.mx)/(pscore.mx*(1-pscore.x)))
+        y0m0=sum(y*(1-d)/(1-pscore.x))/sum((1-d)/(1-pscore.x))
+        y0m1=sum(y*(1-d)*pscore.mx/((1-pscore.mx)*pscore.x))/sum((1-d)*pscore.mx/((1-pscore.mx)*pscore.x))
+      }
+      if (ATET==TRUE){
+        y1m1=sum(y*d)/sum(d)
+        y1m0=sum(y*d*(1-pscore.mx)*pscore.x/(pscore.mx*(1-pscore.x)))/sum(d*(1-pscore.mx)*pscore.x/(pscore.mx*(1-pscore.x)))
+        y0m0=sum(y*(1-d)*pscore.x/(1-pscore.x))/sum((1-d)*pscore.x/(1-pscore.x))
+        y0m1=sum(y*(1-d)*pscore.mx/((1-pscore.mx)))/sum((1-d)*pscore.mx/((1-pscore.mx)))
+      }
     }
-    ind=((pscore.mx<trim) | (pscore.mx>(1-trim)) )
-    y=y[ind==0]; d=d[ind==0]; pscore.mx=pscore.mx[ind==0]; pscore.x=pscore.x[ind==0]
 
-    if (ATET==FALSE){
-      y1m1=sum(y*d/pscore.x)/sum(d/pscore.x)
-      y1m0=sum(y*d*(1-pscore.mx)/(pscore.mx*(1-pscore.x)))/sum(d*(1-pscore.mx)/(pscore.mx*(1-pscore.x)))
-      y0m0=(sum(y*(1-d)/(1-pscore.x))/sum((1-d)/(1-pscore.x)))
-      y0m1=(sum(y*(1-d)*pscore.mx/((1-pscore.mx)*pscore.x))/sum((1-d)*pscore.mx/((1-pscore.mx)*pscore.x)))
-    }
-    if (ATET==TRUE){
-      y1m1=sum(y*d)/sum(d)
-      y1m0=sum(y*d*(1-pscore.mx)*pscore.x/(pscore.mx*(1-pscore.x)))/sum(d*(1-pscore.mx)*pscore.x/(pscore.mx*(1-pscore.x)))
-      y0m0=(sum(y*(1-d)*pscore.x/(1-pscore.x))/sum((1-d)*pscore.x/(1-pscore.x)))
-      y0m1=(sum(y*(1-d)*pscore.mx/((1-pscore.mx)))/sum((1-d)*pscore.mx/((1-pscore.mx))))
+   if (is.null(s)==FALSE & selpop==FALSE){
+      if (ATET==FALSE){
+        y1m1=sum(y*d*s/(pscore.s*pscore.x))/sum(d*s/(pscore.s*pscore.x))
+        y1m0=sum(y*d*s*(1-pscore.mx)/(pscore.s*pscore.mx*(1-pscore.x)))/sum(d*s*(1-pscore.mx)/(pscore.s*pscore.mx*(1-pscore.x)))
+        y0m0=sum(y*(1-d)*s/(pscore.s*(1-pscore.x)))/sum((1-d)*s/(pscore.s*(1-pscore.x)))
+        y0m1=sum(y*(1-d)*s*pscore.mx/(pscore.s*(1-pscore.mx)*pscore.x))/sum((1-d)*s*pscore.mx/(pscore.s*(1-pscore.mx)*pscore.x))
+      }
+      if (ATET==TRUE){
+        y1m1=sum(y*d*s/pscore.s)/sum(d*s/pscore.s)
+        y1m0=sum(y*d*s*(1-pscore.mx)*pscore.x/(pscore.s*pscore.mx*(1-pscore.x)))/sum(d*s*(1-pscore.mx)*pscore.x/(pscore.s*pscore.mx*(1-pscore.x)))
+        y0m0=sum(y*(1-d)*s*pscore.x/(pscore.s*(1-pscore.x)))/sum((1-d)*s*pscore.x/(pscore.s*(1-pscore.x)))
+        y0m1=sum(y*(1-d)*s*pscore.mx/(pscore.s*(1-pscore.mx)))/sum((1-d)*s*pscore.mx/(pscore.s*(1-pscore.mx)))
+      }
     }
     results=c(y1m1-y0m0, y1m1 - y0m1, y1m0-y0m0, y1m1 - y1m0, y0m1 - y0m0, sum(ind))
   }
@@ -64,14 +103,16 @@ mediation<-function(y,d,m,x,w=NULL,trim=0.05, ATET=FALSE, logit=FALSE){
   results
 }
 
-bootstrap.mediation<-function(y,d,m,x,w=NULL,boot=1999,trim=0.05, ATET=FALSE, logit=FALSE, cluster=NULL){
+bootstrap.mediation<-function(y,d,m,x,w=NULL,s=NULL,z=NULL,boot=1999, selpop=FALSE, trim=0.05, ATET=FALSE, logit=FALSE, cluster=NULL){
   if (is.null(cluster)){
     obs<-length(y)
     bsamples=matrix( ,boot,6)
     for(i in 1:boot){
       sboot<-sample(1:obs,obs,TRUE)
       yb=y[sboot]
-      db<-d[sboot]
+      db=d[sboot]
+      if (is.null(s)==FALSE) sb=s[sboot]
+      if (is.null(s)==TRUE) sb=NULL
       if (is.null(ncol(m))) mb<-m[sboot]
       if (is.null(ncol(m))==0) mb<-m[sboot,]
       if (is.null(ncol(x))) xb<-x[sboot]
@@ -79,7 +120,11 @@ bootstrap.mediation<-function(y,d,m,x,w=NULL,boot=1999,trim=0.05, ATET=FALSE, lo
       if ( (is.null(w)==FALSE) & (length(w)==length(y))) wb=w[sboot]
       if ( (is.null(w)==FALSE) & (length(w)!=length(y))) wb=w[sboot,]
       if (is.null(w)==TRUE) wb=NULL
-      bsamples[i,]=c(mediation(y=yb,d=db,m=mb,x=xb,w=wb, trim=trim, ATET=ATET, logit=logit))
+      if ( (is.null(z)==FALSE) & (length(z)==length(y))) zb=z[sboot]
+      if ( (is.null(z)==FALSE) & (length(z)!=length(y))) zb=z[sboot,]
+      if (is.null(z)==TRUE) zb=NULL
+
+      bsamples[i,]=c(mediation(y=yb,d=db,m=mb,x=xb,w=wb, s=sb, z=zb, selpop=selpop, trim=trim, ATET=ATET, logit=logit))
     }
   }
   if (is.null(cluster)==0){
@@ -91,18 +136,23 @@ bootstrap.mediation<-function(y,d,m,x,w=NULL,boot=1999,trim=0.05, ATET=FALSE, lo
     obs<-length(clusters)
     while(length(temp)<boot){
       sboot<-sample(clusters,obs,TRUE)
-      db<-c(); yb<-c(); xb<-c() ; mb=c(); wb=c()
+      db<-c(); yb<-c(); xb<-c() ; mb=c(); wb=c(); sb=c(); zb=c()
       for (k in 1:length(sboot)) {
         db<-c(db,d[key==sboot[k]]); yb<-c(yb,y[key==sboot[k]])
+        if (is.null(s)==FALSE) sb<-c(sb,s[key==sboot[k]])
         if (is.null(ncol(m))) mb<-c(mb,m[key==sboot[k]])
         if (is.null(ncol(m))==0) mb=rbind(mb,m[key==sboot[k],])
         if (is.null(ncol(x))) xb<-c(xb,x[key==sboot[k]])
         if (is.null(ncol(x))==0) xb=rbind(xb,x[key==sboot[k],])
         if ((is.null(w)==FALSE) & is.null(ncol(w))) wb<-c(wb,w[key==sboot[k]])
         if ((is.null(w)==FALSE) & is.null(ncol(w))==0) wb=rbind(wb,w[key==sboot[k],])
+        if ((is.null(z)==FALSE) & is.null(ncol(z))) zb<-c(zb,z[key==sboot[k]])
+        if ((is.null(z)==FALSE) & is.null(ncol(z))==0) zb=rbind(zb,z[key==sboot[k],])
       }
       if (is.null(w)==TRUE) wb=NULL
-      est=c(mediation(y=yb,d=db,m=mb,x=xb,w=wb, trim=trim, ATET=ATET, logit=logit))
+      if (is.null(s)==TRUE) sb=NULL
+      if (is.null(z)==TRUE) zb=NULL
+      est=c(mediation(y=yb,d=db,m=mb,x=xb,w=wb, s=sb, z=zb, trim=trim, ATET=ATET, logit=logit))
       bsamples<-rbind(bsamples, est)
       temp<-c(temp,1)
     }
@@ -115,8 +165,7 @@ bootstrap.mediation<-function(y,d,m,x,w=NULL,boot=1999,trim=0.05, ATET=FALSE, lo
 
 
 ipw<-function(y,d,x,s,z, selpop=FALSE, trim=0.05, ATET=FALSE, logit=FALSE){
-  if (is.null(s)==0) y[s==0]=0
-    if (logit==FALSE) {
+   if (logit==FALSE) {
       if ( (is.null(s))  | (is.null(s)==0 & is.null(z)) )  pscore.x=glm(d~x,family=binomial(probit))$fitted
       if (is.null(s)==0 & is.null(z)) selscore=glm(s~cbind(d,x),family=binomial(probit))$fitted
       if (is.null(s)==0 & is.null(z)==0) {
@@ -418,11 +467,11 @@ mediation.cont<-function(y,d,m,x, d0, d1, ATET=FALSE, trim=0.05, lognorm=FALSE,
     dd=d;dd[d==0]=0.00001
     ggg=glm(log(dd)~x)
     if (d0==0) d0=0.00001; if (d1==0) d1=0.00001
-    pscore1d0=(dnorm( (log(d0)-cbind(1,x)%*%ggg$coefficients)/sqrt(mean(ggg$residuals^2))))
-    pscore1d1=(dnorm( (log(d1)-cbind(1,x)%*%ggg$coefficients)/sqrt(mean(ggg$residuals^2))))
+    pscore1d0=(dnorm( (log(d0)-cbind(1,x)%*%ggg$coefficients)/sqrt(mean(ggg$residuals^2)))/d0)
+    pscore1d1=(dnorm( (log(d1)-cbind(1,x)%*%ggg$coefficients)/sqrt(mean(ggg$residuals^2)))/d1)
     ggg=glm(log(dd)~cbind(x,m))
-    pscore2d0=(dnorm( (log(d0)-cbind(1,x,m)%*%ggg$coefficients)/sqrt(mean(ggg$residuals^2))))
-    pscore2d1=(dnorm( (log(d1)-cbind(1,x,m)%*%ggg$coefficients)/sqrt(mean(ggg$residuals^2))))
+    pscore2d0=(dnorm( (log(d0)-cbind(1,x,m)%*%ggg$coefficients)/sqrt(mean(ggg$residuals^2)))/d0)
+    pscore2d1=(dnorm( (log(d1)-cbind(1,x,m)%*%ggg$coefficients)/sqrt(mean(ggg$residuals^2)))/d1)
   }
   if(lognorm==FALSE){
     ggg=glm(d~x)
@@ -434,7 +483,8 @@ mediation.cont<-function(y,d,m,x, d0, d1, ATET=FALSE, trim=0.05, lognorm=FALSE,
   }
   kernwgtd0=npksum(bws=bw, txdat = d, tydat = y, exdat = d0, return.kernel.weights=TRUE, ckertype="epanechnikov", ckerorder=2)$kw
   kernwgtd1=npksum(bws=bw, txdat = d, tydat = y, exdat = d1, return.kernel.weights=TRUE, ckertype="epanechnikov", ckerorder=2)$kw
-  ind=((pscore2d0>=trim) & (pscore2d1>=trim) )
+  if(lognorm==FALSE) ind=((pscore2d0>=trim) & (pscore2d1>=trim) )
+  if(lognorm==TRUE)  ind=((pscore2d0>=quantile(pscore2d0, trim)) & (pscore2d1>=quantile(pscore2d1, trim)) )
   y=y[ind];  pscore1d0=pscore1d0[ind];pscore1d1=pscore1d1[ind];pscore2d0=pscore2d0[ind]; pscore2d1=pscore2d1[ind];
   kernwgtd0=kernwgtd0[ind]; kernwgtd1= kernwgtd1[ind]
   if (ATET==FALSE){
